@@ -392,6 +392,8 @@ size_t Archive::ReadHeader15()
               byte CurByte=Raw.Get1();
               rlt.Reminder|=(((uint)CurByte)<<((J+3-count)*8));
             }
+            // Convert from 100ns RAR precision to REMINDER_PRECISION.
+            rlt.Reminder*=RarTime::REMINDER_PRECISION/10000000;
             CurTime->SetLocal(&rlt);
           }
         }
@@ -562,7 +564,7 @@ size_t Archive::ReadHeader50()
 
   if (Decrypt)
   {
-#if defined(SHELL_EXT) || defined(RAR_NOCRYPT)
+#if defined(RAR_NOCRYPT)
     return 0;
 #else
     RequestArcPassword();
@@ -772,7 +774,7 @@ size_t Archive::ReadHeader50()
         hd->MaxSize=Max(hd->PackSize,hd->UnpSize);
         hd->FileAttr=(uint)Raw.GetV();
         if ((hd->FileFlags & FHFL_UTIME)!=0)
-          hd->mtime=(time_t)Raw.Get4();
+          hd->mtime.SetUnix((time_t)Raw.Get4());
 
         hd->FileHash.Type=HASH_NONE;
         if ((hd->FileFlags & FHFL_CRC32)!=0)
@@ -835,6 +837,20 @@ size_t Archive::ReadHeader50()
         if (!FileBlock && hd->CmpName(SUBHEAD_TYPE_CMT))
           MainComment=true;
 
+#if 0
+        // For RAR5 format we read the user specified recovery percent here.
+        // It would be useful to do it for shell extension too, so we display
+        // the correct recovery record size in archive properties. But then
+        // we would need to include the entire recovery record processing
+        // code to shell extension, which is not done now.
+        if (!FileBlock && hd->CmpName(SUBHEAD_TYPE_RR) && hd->SubData.Size()>0)
+        {
+          RecoveryPercent=hd->SubData[0];
+          RSBlockHeader Header;
+          GetRRInfo(this,&Header);
+          RecoverySize=Header.RecSectionSize*Header.RecCount;
+        }
+#endif
           
         if (BadCRC) // Add the file name to broken header message displayed above.
           uiMsg(UIERROR_FHEADERBROKEN,Archive::FileName,hd->FileName);
@@ -861,7 +877,7 @@ size_t Archive::ReadHeader50()
 }
 
 
-#if !defined(SHELL_EXT) && !defined(RAR_NOCRYPT)
+#if !defined(RAR_NOCRYPT)
 void Archive::RequestArcPassword()
 {
   if (!Cmd->Password.IsSet())
@@ -1012,25 +1028,35 @@ void Archive::ProcessExtra50(RawRead *Raw,size_t ExtraSize,BaseBlock *bb)
           }
           break;
         case FHEXTRA_HTIME:
-          if (FieldSize>=9)
+          if (FieldSize>=5)
           {
             byte Flags=(byte)Raw->GetV();
             bool UnixTime=(Flags & FHEXTRA_HTIME_UNIXTIME)!=0;
             if ((Flags & FHEXTRA_HTIME_MTIME)!=0)
               if (UnixTime)
-                hd->mtime=(time_t)Raw->Get4();
+                hd->mtime.SetUnix(Raw->Get4());
               else
-                hd->mtime.SetRaw(Raw->Get8());
+                hd->mtime.SetWin(Raw->Get8());
             if ((Flags & FHEXTRA_HTIME_CTIME)!=0)
               if (UnixTime)
-                hd->ctime=(time_t)Raw->Get4();
+                hd->ctime.SetUnix(Raw->Get4());
               else
-                hd->ctime.SetRaw(Raw->Get8());
+                hd->ctime.SetWin(Raw->Get8());
             if ((Flags & FHEXTRA_HTIME_ATIME)!=0)
               if (UnixTime)
-                hd->atime=(time_t)Raw->Get4();
+                hd->atime.SetUnix((time_t)Raw->Get4());
               else
-                hd->atime.SetRaw(Raw->Get8());
+                hd->atime.SetWin(Raw->Get8());
+            if (UnixTime && (Flags & FHEXTRA_HTIME_UNIX_NS)!=0) // Add nanoseconds.
+            {
+              uint ns;
+              if ((Flags & FHEXTRA_HTIME_MTIME)!=0 && (ns=(Raw->Get4() & 0x3fffffff))<1000000000)
+                hd->mtime.Adjust(ns);
+              if ((Flags & FHEXTRA_HTIME_CTIME)!=0 && (ns=(Raw->Get4() & 0x3fffffff))<1000000000)
+                hd->ctime.Adjust(ns);
+              if ((Flags & FHEXTRA_HTIME_ATIME)!=0 && (ns=(Raw->Get4() & 0x3fffffff))<1000000000)
+                hd->atime.Adjust(ns);
+            }
           }
           break;
         case FHEXTRA_VERSION:
@@ -1343,7 +1369,6 @@ int64 Archive::GetStartPos()
 }
 
 
-#ifndef SHELL_EXT
 bool Archive::ReadSubData(Array<byte> *UnpData,File *DestFile)
 {
   if (BrokenHeader)
@@ -1410,4 +1435,3 @@ bool Archive::ReadSubData(Array<byte> *UnpData,File *DestFile)
   }
   return true;
 }
-#endif
