@@ -24,11 +24,11 @@ void RarVM::Execute(VM_PreparedProgram *Prg)
   memcpy(R,Prg->InitR,sizeof(Prg->InitR));
   if (Prg->Type!=VMSF_NONE)
   {
-    ExecuteStandardFilter(Prg->Type);
+    bool Success=ExecuteStandardFilter(Prg->Type);
     uint BlockSize=Prg->InitR[4] & VM_MEMMASK;
     Prg->FilteredDataSize=BlockSize;
     if (Prg->Type==VMSF_DELTA || Prg->Type==VMSF_RGB || Prg->Type==VMSF_AUDIO)
-      Prg->FilteredData=2*BlockSize>=VM_MEMSIZE ? Mem:Mem+BlockSize;
+      Prg->FilteredData=2*BlockSize>VM_MEMSIZE || !Success ? Mem:Mem+BlockSize;
     else
       Prg->FilteredData=Mem;
   }
@@ -111,7 +111,7 @@ void RarVM::SetMemory(size_t Pos,byte *Data,size_t DataSize)
 }
 
 
-void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
+bool RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
 {
   switch(FilterType)
   {
@@ -119,29 +119,31 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
     case VMSF_E8E9:
       {
         byte *Data=Mem;
-        int DataSize=R[4];
-        uint FileOffset=R[6];
+        uint DataSize=R[4],FileOffset=R[6];
 
-        if ((uint)DataSize>=VM_MEMSIZE || DataSize<4)
-          break;
+        if (DataSize>VM_MEMSIZE || DataSize<4)
+          return false;
 
-        const int FileSize=0x1000000;
+        const uint FileSize=0x1000000;
         byte CmpByte2=FilterType==VMSF_E8E9 ? 0xe9:0xe8;
-        for (int CurPos=0;CurPos<DataSize-4;)
+        for (uint CurPos=0;CurPos<DataSize-4;)
         {
           byte CurByte=*(Data++);
           CurPos++;
           if (CurByte==0xe8 || CurByte==CmpByte2)
           {
-            int32 Offset=CurPos+FileOffset;
-            int32 Addr=RawGet4(Data);
-            if (Addr<0)
+            uint Offset=CurPos+FileOffset;
+            uint Addr=RawGet4(Data);
+
+            // We check 0x80000000 bit instead of '< 0' comparison
+            // not assuming int32 presence or uint size and endianness.
+            if ((Addr & 0x80000000)!=0)              // Addr<0
             {
-              if (Addr+Offset>=0)
+              if (((Addr+Offset) & 0x80000000)==0)   // Addr+Offset>=0
                 RawPut4(Addr+FileSize,Data);
             }
             else
-              if (Addr<FileSize)
+              if (((Addr-FileSize) & 0x80000000)!=0) // Addr<FileSize
                 RawPut4(Addr-Offset,Data);
             Data+=4;
             CurPos+=4;
@@ -152,13 +154,12 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
     case VMSF_ITANIUM:
       {
         byte *Data=Mem;
-        int DataSize=R[4];
-        uint FileOffset=R[6];
+        uint DataSize=R[4],FileOffset=R[6];
 
-        if ((uint)DataSize>=VM_MEMSIZE || DataSize<21)
-          break;
+        if (DataSize>VM_MEMSIZE || DataSize<21)
+          return false;
 
-        int CurPos=0;
+        uint CurPos=0;
 
         FileOffset>>=4;
 
@@ -170,14 +171,14 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
             static byte Masks[16]={4,4,6,6,0,0,7,7,4,4,0,0,4,4,0,0};
             byte CmdMask=Masks[Byte];
             if (CmdMask!=0)
-              for (int I=0;I<=2;I++)
+              for (uint I=0;I<=2;I++)
                 if (CmdMask & (1<<I))
                 {
-                  int StartPos=I*41+5;
-                  int OpType=FilterItanium_GetBits(Data,StartPos+37,4);
+                  uint StartPos=I*41+5;
+                  uint OpType=FilterItanium_GetBits(Data,StartPos+37,4);
                   if (OpType==5)
                   {
-                    int Offset=FilterItanium_GetBits(Data,StartPos+13,20);
+                    uint Offset=FilterItanium_GetBits(Data,StartPos+13,20);
                     FilterItanium_SetBits(Data,(Offset-FileOffset)&0xfffff,StartPos+13,20);
                   }
                 }
@@ -190,38 +191,37 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
       break;
     case VMSF_DELTA:
       {
-        int DataSize=R[4],Channels=R[0],SrcPos=0,Border=DataSize*2;
-        if ((uint)DataSize>=VM_MEMSIZE/2)
-          break;
+        uint DataSize=R[4],Channels=R[0],SrcPos=0,Border=DataSize*2;
+        if (DataSize>VM_MEMSIZE/2 || Channels>MAX3_UNPACK_CHANNELS || Channels==0)
+          return false;
 
         // Bytes from same channels are grouped to continual data blocks,
         // so we need to place them back to their interleaving positions.
-        for (int CurChannel=0;CurChannel<Channels;CurChannel++)
+        for (uint CurChannel=0;CurChannel<Channels;CurChannel++)
         {
           byte PrevByte=0;
-          for (int DestPos=DataSize+CurChannel;DestPos<Border;DestPos+=Channels)
+          for (uint DestPos=DataSize+CurChannel;DestPos<Border;DestPos+=Channels)
             Mem[DestPos]=(PrevByte-=Mem[SrcPos++]);
         }
       }
       break;
     case VMSF_RGB:
       {
-        int DataSize=R[4],Width=R[0]-3,PosR=R[1];
+        uint DataSize=R[4],Width=R[0]-3,PosR=R[1];
+        if (DataSize>VM_MEMSIZE/2 || DataSize<3 || Width>DataSize || PosR>2)
+          return false;
         byte *SrcData=Mem,*DestData=SrcData+DataSize;
-        const int Channels=3;
-        if ((uint)DataSize>=VM_MEMSIZE/2 || Width<0 || PosR<0)
-          break;
-        for (int CurChannel=0;CurChannel<Channels;CurChannel++)
+        const uint Channels=3;
+        for (uint CurChannel=0;CurChannel<Channels;CurChannel++)
         {
           uint PrevByte=0;
 
-          for (int I=CurChannel;I<DataSize;I+=Channels)
+          for (uint I=CurChannel;I<DataSize;I+=Channels)
           {
             uint Predicted;
-            int UpperPos=I-Width;
-            if (UpperPos>=3)
+            if (I>=Width+3)
             {
-              byte *UpperData=DestData+UpperPos;
+              byte *UpperData=DestData+I-Width;
               uint UpperByte=*UpperData;
               uint UpperLeftByte=*(UpperData-3);
               Predicted=PrevByte+UpperByte-UpperLeftByte;
@@ -241,7 +241,7 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
             DestData[I]=PrevByte=(byte)(Predicted-*(SrcData++));
           }
         }
-        for (int I=PosR,Border=DataSize-2;I<Border;I+=3)
+        for (uint I=PosR,Border=DataSize-2;I<Border;I+=3)
         {
           byte G=DestData[I+1];
           DestData[I]+=G;
@@ -251,18 +251,19 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
       break;
     case VMSF_AUDIO:
       {
-        int DataSize=R[4],Channels=R[0];
+        uint DataSize=R[4],Channels=R[0];
         byte *SrcData=Mem,*DestData=SrcData+DataSize;
-        if ((uint)DataSize>=VM_MEMSIZE/2)
-          break;
-        for (int CurChannel=0;CurChannel<Channels;CurChannel++)
+        // In fact, audio channels never exceed 4.
+        if (DataSize>VM_MEMSIZE/2 || Channels>128 || Channels==0)
+          return false;
+        for (uint CurChannel=0;CurChannel<Channels;CurChannel++)
         {
           uint PrevByte=0,PrevDelta=0,Dif[7];
           int D1=0,D2=0,D3;
           int K1=0,K2=0,K3=0;
           memset(Dif,0,sizeof(Dif));
 
-          for (int I=CurChannel,ByteCount=0;I<DataSize;I+=Channels,ByteCount++)
+          for (uint I=CurChannel,ByteCount=0;I<DataSize;I+=Channels,ByteCount++)
           {
             D3=D2;
             D2=PrevDelta-D1;
@@ -292,7 +293,7 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
             {
               uint MinDif=Dif[0],NumMinDif=0;
               Dif[0]=0;
-              for (int J=1;J<sizeof(Dif)/sizeof(Dif[0]);J++)
+              for (uint J=1;J<ASIZE(Dif);J++)
               {
                 if (Dif[J]<MinDif)
                 {
@@ -316,13 +317,14 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
       }
       break;
   }
+  return true;
 }
 
 
-uint RarVM::FilterItanium_GetBits(byte *Data,int BitPos,int BitCount)
+uint RarVM::FilterItanium_GetBits(byte *Data,uint BitPos,uint BitCount)
 {
-  int InAddr=BitPos/8;
-  int InBit=BitPos&7;
+  uint InAddr=BitPos/8;
+  uint InBit=BitPos&7;
   uint BitField=(uint)Data[InAddr++];
   BitField|=(uint)Data[InAddr++] << 8;
   BitField|=(uint)Data[InAddr++] << 16;
@@ -332,10 +334,10 @@ uint RarVM::FilterItanium_GetBits(byte *Data,int BitPos,int BitCount)
 }
 
 
-void RarVM::FilterItanium_SetBits(byte *Data,uint BitField,int BitPos,int BitCount)
+void RarVM::FilterItanium_SetBits(byte *Data,uint BitField,uint BitPos,uint BitCount)
 {
-  int InAddr=BitPos/8;
-  int InBit=BitPos&7;
+  uint InAddr=BitPos/8;
+  uint InBit=BitPos&7;
   uint AndMask=0xffffffff>>(32-BitCount);
   AndMask=~(AndMask<<InBit);
 
