@@ -13,9 +13,19 @@
 // from two data blocks.
 #define MAX3_UNPACK_FILTERS      8192
 
+// Limit maximum number of channels in RAR3 delta filter to some reasonable
+// value to prevent too slow processing of corrupt archives with invalid
+// channels number. Must be equal or larger than v3_MAX_FILTER_CHANNELS.
+// No need to provide it for RAR5, which uses only 5 bits to store channels.
+#define MAX3_UNPACK_CHANNELS      1024
+
+// Maximum size of single filter block. We restrict it to limit memory
+// allocation. Must be equal or larger than MAX_ANALYZE_SIZE.
+#define MAX_FILTER_BLOCK_SIZE 0x400000
+
 // Write data in 4 MB or smaller blocks. Must not exceed PACK_MAX_WRITE,
 // so we keep number of buffered filter in unpacker reasonable.
-#define UNPACK_MAX_WRITE     0x400000
+#define UNPACK_MAX_WRITE      0x400000
 
 // Decode compressed bit fields to alphabet numbers.
 struct DecodeTable:PackDef
